@@ -1,11 +1,11 @@
 # Compartmented models base class
 #
-# Copyright (C) 2017--2020 Simon Dobson
-# 
+# Copyright (C) 2017--2021 Simon Dobson
+#
 # This file is part of epydemic, epidemic network simulations in Python.
 #
 # epydemic is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
+# it under the terms of the GNU General Public License as published byf
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
@@ -17,17 +17,17 @@
 # You should have received a copy of the GNU General Public License
 # along with epydemic. If not, see <http://www.gnu.org/licenses/gpl.html>.
 
-from epydemic import Locus, Process, Node, Edge, Element
-from networkx import Graph
-import math
-import numpy
 import sys
+import math
+from networkx import Graph
+import numpy
+from typing import Dict, Any, List, Tuple, Callable
 if sys.version_info >= (3, 8):
-    from typing import Dict, Any, List, Final, Tuple, Callable
+    from typing import Final
 else:
     # backport compatibility with older typing
-    from typing import Dict, Any, List, Tuple, Callable
     from typing_extensions import Final
+from epydemic import Locus, Process, Node, Edge, Element
 
 # Helper types
 Handlers = Tuple[Callable[[Graph, Element], None],   # add handler
@@ -35,13 +35,14 @@ Handlers = Tuple[Callable[[Graph, Element], None],   # add handler
                  Callable[[Graph, Element], None],   # enter handler
                  Callable[[Graph, Element], None]]   # remove handler
 
+
 class CompartmentedLocus(Locus):
     '''A locus based on the compartments that nodes reside in.
 
     :param name: the locus' name'''
 
     def __init__(self, name : str):
-        super(CompartmentedLocus, self).__init__(name)
+        super().__init__(name)
 
     def compartments(self) -> List[str]:
         '''Return the compartments this locus monitors. This should be
@@ -49,7 +50,7 @@ class CompartmentedLocus(Locus):
 
         :returns: the compartments'''
         return []
-   
+
 
 class CompartmentedNodeLocus(CompartmentedLocus):
     '''A locus for dynamics occurring at a single node, where the
@@ -58,47 +59,47 @@ class CompartmentedNodeLocus(CompartmentedLocus):
     :param name: the locus' name
     :param c: the compartment'''
 
-    def __init__(self, name : str, c : str):
-        super(CompartmentedNodeLocus, self).__init__(name)
+    def __init__(self, name: str, c: str):
+        super().__init__(name)
         self._compartment = c
 
     def compartments(self) -> List[str]:
         '''Return the node compartment we monitor.
 
         :returns: the compartment'''
-        return [ self._compartment ]
+        return [self._compartment]
 
-    def addHandler(self, g : Graph, n : Element):
-        '''A node is added to the network. 
+    def addHandler(self, g: Graph, n: Element):
+        '''A node is added to the network.
 
         :param, g: the network
         :param n: the node'''
         if not isinstance(n, tuple):
-            super(CompartmentedNodeLocus, self).addHandler(g, n)
+            super().addHandler(g, n)
 
-    def leaveHandler(self, g : Graph, n : Element):
-        '''A node changes compartment. 
+    def leaveHandler(self, g: Graph, n: Element):
+        '''A node changes compartment.
 
         :param, g: the network
         :param n: the node'''
         if not isinstance(n, tuple):
-            super(CompartmentedNodeLocus, self).leaveHandler(g, n)
+            super().leaveHandler(g, n)
 
-    def enterHandler(self, g : Graph, n : Element):
-        '''A node enters a compartment. 
+    def enterHandler(self, g: Graph, n: Element):
+        '''A node enters a compartment.
 
         :param, g: the network
         :param n: the node'''
         if not isinstance(n, tuple):
-            super(CompartmentedNodeLocus, self).enterHandler(g, n)
+            super().enterHandler(g, n)
 
-    def removeHandler(self, g : Graph, n : Element):
-        '''A node is removed from the network. 
+    def removeHandler(self, g: Graph, n: Element):
+        '''A node is removed from the network.
 
         :param, g: the network
         :param n: the node'''
         if not isinstance(n, tuple):
-            super(CompartmentedNodeLocus, self).removeHandler(g, n)
+            super().removeHandler(g, n)
 
 
 class CompartmentedEdgeLocus(CompartmentedLocus):
@@ -110,8 +111,8 @@ class CompartmentedEdgeLocus(CompartmentedLocus):
     :param l: the left compartment
     :param r: the right compartment'''
 
-    def __init__(self, name : str, l : str, r : str):
-        super(CompartmentedEdgeLocus, self).__init__(name)
+    def __init__(self, name: str, l: str, r: str):
+        super().__init__(name)
         self._left = l
         self._right = r
 
@@ -119,9 +120,9 @@ class CompartmentedEdgeLocus(CompartmentedLocus):
         '''Return the compartments of the node endpoints we monitor.
 
         :returns: the compartments'''
-        return [ self._left, self._right ]
+        return [self._left, self._right]
 
-    def matches(self, g : Graph, n : Node, m : Node) -> int:
+    def matches(self, g: Graph, n :Node, m: Node) -> int:
         '''Test whether the given edge has the right compartment endpoints for this compartment. The
         method returns 1 if the edge has the right compartments in the orientation (n, m), -1 if it has
         the right compartments in orientation (m, n), and 0 otherwise.
@@ -138,59 +139,56 @@ class CompartmentedEdgeLocus(CompartmentedLocus):
             else:
                 return 0
 
-    def addHandler(self, g : Graph, e : Edge):
+    def addHandler(self, g: Graph, e: Edge):
         '''An edge is added to the network, check if its endpoint compartments match the
-        locus and add it if so. 
+        locus and add it if so.
 
         :param, g: the network
         :param e: the edge'''
         if isinstance(e, tuple):
             (n, m) = e
-            es = self.elements()
             match = self.matches(g, n, m)
             if match == -1:
                 #print('edge ({m}, {n}) added to {l}'.format(n = n, m = m, l = self._name))
-                es.add((m, n))
+                self.add((m, n))
             else:
                 if match == 1:
                     #print('edge ({n}, {m}) added {l}'.format(n = n, m = m, l = self._name))
-                    es.add((n, m))
+                    self.add((n, m))
 
-    def leaveHandler(self, g : Graph, n : Node):
+    def leaveHandler(self, g: Graph, n: Node):
         '''Node leaves one of the edge's compartments, remove any incident edges
         that no longer have the correct orientation.
 
         :param g: the network
         :param n: the node'''
-        es = self.elements()
         for (nn, mm) in g.edges(n):
             match = self.matches(g, nn, mm)
             if match == -1:
                 #print('edge ({m}, {n}) leaves {l}'.format(n = nn, m = mm, l = self._name))
-                es.discard((mm, nn))
+                self.discard((mm, nn))
             else:
                 if match == 1:
                     #print('edge ({n}, {m}) leaves {l}'.format(n = nn, m = mm, l = self._name))
-                    es.discard((nn, mm))
+                    self.discard((nn, mm))
 
-    def enterHandler(self, g : Graph, n : Node):
+    def enterHandler(self, g: Graph, n: Node):
         '''Node enters one of the edge's compartments, add any incident edges
         that now have the correct orientation.
 
         :param g: the network
         :param n: the node'''
-        es = self.elements()
         for (nn, mm) in g.edges(n):
             match = self.matches(g, nn, mm)
             if match == -1:
                 #print('edge ({m}, {n}) enters {l}'.format(n = nn, m = mm, l = self._name))
-                es.add((mm, nn))
+                self.add((mm, nn))
             else:
                 if match == 1:
                     #print('edge ({n}, {m}) enters {l}'.format(n = nn, m = mm, l = self._name))
-                    es.add((nn, mm))
+                    self.add((nn, mm))
 
-    def removeHandler(self, g : Graph, e : Edge):
+    def removeHandler(self, g: Graph, e: Edge):
         '''An edge is removed from the network, check whether it was in this locus and
         remove it if so.
 
@@ -198,15 +196,14 @@ class CompartmentedEdgeLocus(CompartmentedLocus):
         :param e: the edge'''
         if isinstance(e, tuple):
             (n, m) = e
-            es = self.elements()
             match = self.matches(g, n, m)
             if match == -1:
                 #print('edge ({m}, {n}) removed from {l}'.format(n = n, m = m, l = self._name))
-                es.discard((m, n))
+                self.discard((m, n))
             else:
                 if match == 1:
                     #print('edge ({n}, {m}) removed from {l}'.format(n = n, m = m, l = self._name))
-                    es.discard((n, m))
+                    self.discard((n, m))
 
 
 class CompartmentedModel(Process):
@@ -220,31 +217,31 @@ class CompartmentedModel(Process):
     When run, a compartmented model generates results consisting of the size of each
     compartment at the end of the experiment. This can be changed by overriding the
     :meth:`results` method.'''
-    
+
     # model state variables
     COMPARTMENT : Final[str] = 'compartment'                 #: Node attribute holding the node's compartment.
     OCCUPIED : Final[str] = 'occupied'                       #: Edge attribute, True if infection travelled along the edge.
     T_OCCUPIED : Final[str] = 'occupationTime'               #: Edge attribute holding the time the infection crossed the edge.
 
-    def __init__( self ):
-        super(CompartmentedModel, self).__init__()
-        self._compartments : Dict[str, float] = dict()         # compartment -> initial probability
-        self._effects : Dict[str, List[Handlers]] = dict()     # compartment -> event handlers
+    def __init__(self):
+        super().__init__()
+        self._compartments: Dict[str, float] = dict()         # compartment -> initial probability
+        self._effects: Dict[str, List[Handlers]] = dict()     # compartment -> event handlers
 
 
     # ---------- Setup and initialisation ----------
 
     def reset(self):
         '''Reset the model ready to be built.'''
-        super(CompartmentedModel, self).reset()
-        self._compartments : Dict[str, float] = dict()
-        self._effects : Dict[str, List[Handlers]] = dict()
+        super().reset()
+        self._compartments = dict()
+        self._effects = dict()
 
-    def setUp(self, params : Dict[str, Any]):
+    def setUp(self, params: Dict[str, Any]):
         '''Set up the initial population of nodes into compartments.
 
         :param params: the simulation parameters'''
-        super(CompartmentedModel, self).setUp(params)
+        super().setUp(params)
 
         # initialise all nodes to an empty compartment
         # (so we can assume all nodes have a compartment attribute)
@@ -262,20 +259,21 @@ class CompartmentedModel(Process):
     def initialCompartmentDistribution(self) -> List[Tuple[str, float]]:
         '''Return the initial distribution of nodes to compartments. The
         result should be a valid distribution, with probabilities summing
-        to one. This is used by :meth:`initialCompartments` to set the initial
+        to one. (A ``ValueError`` exception is raised if not.) The
+        distribution is used by :meth:`initialCompartments` to set the initial
         compartment of each node.
 
         :returns: a list of (compartment, probability) pairs'''
         dist = []
         for cp in self._compartments.items():
             dist.append(cp)
- 
+
         # sanity-check the distribution
         a = 0.0
         for (_, p) in dist:
             a += p
         if not math.isclose(a, 1.0):
-            raise Exception('Bad initial compartment distribution (probabilities don\'t sum to one')
+            raise ValueError('Bad initial compartment distribution (probabilities don\'t sum to one)')
 
         return dist
 
@@ -290,8 +288,8 @@ class CompartmentedModel(Process):
         dist = self.initialCompartmentDistribution()
 
         # assign nodes to compartments
-        g = self.network()
         rng = numpy.random.default_rng()
+        g = self.network()
         for n in g.nodes():
             # select a compartment according to the initial distribution
             r = rng.random()
@@ -300,11 +298,22 @@ class CompartmentedModel(Process):
                 a += p
                 if r <= a:
                     # change node's compartment
-                    self.changeCompartment(n, c)
+                    self.changeInitialCompartment(n, c)
 
                     # on to the next node
                     break
 
+    def changeInitialCompartment(self, n: Node, c: str):
+        '''Change the initial compartment of a node. This is called only from
+        :meth:`initialCompartments`, and by default simply calls
+        :meth:`changeCompartment`. Overriding this method changes the behaviour
+        of assigning initial compartments independently of the behaviour
+        of later changes.
+
+        :param n: the node
+        :param c: the new compartment for the node'''
+        self.changeCompartment(n, c)
+
 
     # ---------- Termination and results ----------
 
@@ -314,13 +323,17 @@ class CompartmentedModel(Process):
         :returns: the compartments'''
         return list(self._compartments.keys())
 
-    def compartment(self, c : str) -> List[Any]:
-        '''Return all the nodes currently in a particular compartment in a network. This works
-        for all compartments, not just those that are loci for dynamics -- but is a *lot*
-        slower, so it's better to create a :term:`locus` is you're going to access a compartment frequently.
+    def compartment(self, c: str) -> List[Any]:
+        '''Return all the nodes currently in a particular compartment in a
+        network. This works for all compartments, not just those that
+        are loci for dynamics -- but is a *lot* slower, so it's better
+        to create a :term:`locus` is you're going to access a
+        compartment frequently.
 
         :param c: the compartment
-        :returns: a collection of nodes'''
+        :returns: a collection of nodes
+
+        '''
         return [ n for n in self.network().nodes() if self.getCompartment(n) == c ]
 
     def results(self) -> Dict[str, Any]:
@@ -362,20 +375,20 @@ class CompartmentedModel(Process):
 
     # ---------- Managing compartments ----------
 
-    def addCompartment(self, c : str, p : float =0.0):
-        '''Add a compartment to the model. A node is assigned to the compartment
-        initially with the given probability. The probabilities for all compartments
-        in the model must sum to 1.
+    def addCompartment(self, c: str, p: float = 0.0):
+        '''Add a compartment to the model. A node is assigned to the
+        compartment initially with the given probability. The
+        probabilities for all compartments in the model must sum to 1.
 
         :param c: the compartment name
         :param p: the initial occupancy probability (defaults to  0.0)'''
         self._compartments[c] = p
 
-    def changeCompartmentInitialOccupancy(self, c : str, p : float):
-        '''Change the initial occupancy probability for a compartment. This method
-        is used when sub-classing an existing model: it only makes sense during the
-        build process (see :meth:`build`) before the model is initialised in
-        :meth:`setUp`.
+    def changeCompartmentInitialOccupancy(self, c: str, p: float):
+        '''Change the initial occupancy probability for a compartment. This
+        method is used when sub-classing an existing model: it only
+        makes sense during the build process (see :meth:`build`)
+        before the model is initialised in :meth:`setUp`.
 
         :param c: the compartment
         :param p: the new initial occupation probability'''
@@ -383,7 +396,7 @@ class CompartmentedModel(Process):
             raise Exception('Compartment {c} not defined in model'.format(c=c))
         self._compartments[c] = p
 
-    def trackNodesInCompartment(self, c : str, name : str =None):
+    def trackNodesInCompartment(self, c: str, name: str = None):
         '''Add a locus tracking nodes in a given compartment.
 
         :param c: the compartment to track
@@ -396,14 +409,14 @@ class CompartmentedModel(Process):
         locus = CompartmentedNodeLocus(name, c)
         return self.addLocus(name, locus)
 
-    def trackEdgesBetweenCompartments(self, l : str, r : str, name : str =None):
+    def trackEdgesBetweenCompartments(self, l: str, r: str, name: str = None):
         '''Add a locus to track edges with endpoint nodes in the given compartments.
 
         :param l: the compartment of the left node
         :param r: the compartment of the right node
         :param name: (optional) the name of the locus (defaults to a combination of the two compartment names)
         :returns: the locus used to track the nodes'''
- 
+
         if name is None:
             name = '{l}-{r}'.format(l = l, r = r)
 
@@ -411,13 +424,13 @@ class CompartmentedModel(Process):
         locus = CompartmentedEdgeLocus(name, l, r)
         return self.addLocus(name, locus)
 
-    def addLocus(self, n: str, l : Locus =None):
+    def addLocus(self, n: str, l: Locus = None):
         '''Add a locus to the model, initialising the handler functions.
 
         :param n: the name
         :param l: (optional) the locus
         :returns: the locus'''
-        locus = super(CompartmentedModel,self).addLocus(n, l)
+        locus = super().addLocus(n, l)
 
         # if we've added a compartmented locus, add handler functions for
         # when its population changes
@@ -427,7 +440,7 @@ class CompartmentedModel(Process):
                     self._effects[c] = []
                 self._effects[c].append((locus.addHandler, locus.leaveHandler, locus.enterHandler, locus.removeHandler))
 
-    def _handlerCompartments(self, e : Element) -> List[str]:
+    def _handlerCompartments(self, e: Element) -> List[str]:
         '''Return the compartments that a given element's change might affect.
 
         :param e: a node or edge
@@ -442,7 +455,7 @@ class CompartmentedModel(Process):
             cs = [ g.nodes[e][self.COMPARTMENT] ]
         return cs
 
-    def _callAddHandlers(self, e : Element):
+    def _callAddHandlers(self, e: Element):
         '''Call all handlers affected by a node or edge being added to the network.
 
         :param e the node or edge'''
@@ -452,7 +465,7 @@ class CompartmentedModel(Process):
                 for (ah, _, _, _) in self._effects[c]:
                     ah(g, e)
 
-    def _callLeaveHandlers(self, e : Element, c : str):
+    def _callLeaveHandlers(self, e : Element, c: str):
         '''Call all handlers affected by a node or edge leaving a compartment.
 
         :param e: the node or edge
@@ -463,7 +476,7 @@ class CompartmentedModel(Process):
                 for (_, lh, _, _) in self._effects[c]:
                     lh(g, e)
 
-    def _callEnterHandlers(self, e : Element, c : str):
+    def _callEnterHandlers(self, e: Element, c: str):
         '''Call all handlers affected by a node or edge entering a compartment.
 
         :param e: the node or edge
@@ -472,9 +485,9 @@ class CompartmentedModel(Process):
         for c in self._handlerCompartments(e):
             if c in self._effects.keys():
                 for (_, _, eh, _) in self._effects[c]:
-                    eh(g, e) 
+                    eh(g, e)
 
-    def _callRemoveHandlers(self, e : Element):
+    def _callRemoveHandlers(self, e: Element):
         '''Call all handlers affected by a node or edge being removed from the network.
 
         :param e: the node or edge'''
@@ -487,10 +500,11 @@ class CompartmentedModel(Process):
 
     # ---------- Accessing and evolving the network ----------
 
-    def setCompartment(self, n : Node, c : str):
-        '''Set the compartment of a node. This assumes that the node doesn't already have
-        a compartment set, and so should be used only for initialising new nodes: in all
-        other cases, use :meth:`changeCompartment`.
+    def setCompartment(self, n: Node, c: str):
+        '''Set the compartment of a node. This assumes that the node doesn't
+        already have a compartment set, and so should be used only for
+        initialising new nodes: in all other cases, use
+        :meth:`changeCompartment`.
 
         :param n: the node
         :param c: the new compartment for the node'''
@@ -502,14 +516,14 @@ class CompartmentedModel(Process):
         # propagate the change to any other compartments
         self._callEnterHandlers(n, c)
 
-    def getCompartment(self, n : Node) -> str:
+    def getCompartment(self, n: Node) -> str:
         '''Return the compartment of a node.
 
-        :parak n: the node
+        :param n: the node
         :returns: its compartment'''
         return self.network().nodes[n][self.COMPARTMENT]
 
-    def changeCompartment(self, n : Node, c : str):
+    def changeCompartment(self, n: Node, c: str):
         '''Change the compartment of a node.
 
         :param n: the node
@@ -527,7 +541,7 @@ class CompartmentedModel(Process):
         # propagate effects of entering new compartment
         self._callEnterHandlers(n, c)
 
-    def markOccupied(self, e : Edge, t : float):
+    def markOccupied(self, e: Edge, t: float):
         '''Mark the given edge as having been occupied by the dynamics, i.e., to
         have been traversed in transmitting the disease, at time t.
 
@@ -538,19 +552,19 @@ class CompartmentedModel(Process):
         data = g.get_edge_data(n, m)
         data[self.OCCUPIED] = True
         data[self.T_OCCUPIED] = t
-        
-    def addNode(self, n : Node, c : str =None, **kwds):
+
+    def addNode(self, n: Node, c: str = None, **kwds):
         '''Add a node to the working network, adding it to the appropriate compartment
         if one is provided].
 
         :param n: the new node
         :param c: (optional) compartment for the node
         :param kwds: (optional) node attributes'''
-        super(CompartmentedModel, self).addNode(n, **kwds)
+        super().addNode(n, **kwds)
         if c is not None:
             self.setCompartment(n, c)
 
-    def removeNode(self, n : Node):
+    def removeNode(self, n: Node):
         '''Remove a node from the working network, updating any affected compartments.
 
         :param n: the node'''
@@ -559,20 +573,20 @@ class CompartmentedModel(Process):
         self._callRemoveHandlers(n)
 
         # remove the node
-        super(CompartmentedModel, self).removeNode(n)
+        super().removeNode(n)
 
-    def addEdge(self, n : Node, m : Node, **kwds):
+    def addEdge(self, n: Node, m: Node, **kwds):
         '''Add an edge between nodes, adding the edge to any appropriate compartments.
 
         :param n: the start node
         :param m: the end node
         :param kwds: (optional) edge attributes'''
-        super(CompartmentedModel, self).addEdge(n, m, **kwds)
+        super().addEdge(n, m, **kwds)
 
         # add edge to any compartments it should be in
         self._callAddHandlers((n, m))
 
-    def removeEdge(self, n : Node, m : Node):
+    def removeEdge(self, n: Node, m: Node):
         '''Remove an edge from the working network and from any compartments.
 
         :param n: the start node
@@ -582,4 +596,4 @@ class CompartmentedModel(Process):
         self._callRemoveHandlers((n, m))
 
         # remove the edge from the network
-        super(CompartmentedModel, self).removeEdge(n, m)
+        super().removeEdge(n, m)
