@@ -1,6 +1,6 @@
 # Networks process base class
 #
-# Copyright (C) 2017--2020 Simon Dobson
+# Copyright (C) 2017--2021 Simon Dobson
 #
 # This file is part of epydemic, epidemic network simulations in Python.
 #
@@ -17,9 +17,9 @@
 # You should have received a copy of the GNU General Public License
 # along with epydemic. If not, see <http://www.gnu.org/licenses/gpl.html>.
 
-from epydemic import Locus, Node, Edge, Element
-from networkx import Graph
 from typing import Dict, List, Tuple, Any, Callable, Iterable, Union
+from networkx import Graph
+from epydemic import Locus, Node, Edge, Element
 
 # There is a circular import between Process and Dynamics at the typing level
 # (but not at the execution level), when providing types for dynamics() and
@@ -30,30 +30,37 @@ from typing import TYPE_CHECKING
 if TYPE_CHECKING:
     from epydemic import Dynamics
 
+
 # Types for handling events
 EventFunction = Callable[[float, Element], None]              #: Type of event-handler functions.
 EventDistribution = List[Tuple[Locus, float, EventFunction]]  #: Type of event distributions.
 
-class Process(object):
-    """A process that runs over a network. This is the abstract base class for all network processes.
-    It provides the essential routines to build, set-up, run, and extract results from a
-    process. Sub-classes provide the actual behaviour by defining simulation events and
-    attaching them to the network in different ways.
 
-    A process definition is largely declarative, in the sense that it sets up event handlers
-    and their probabilities. These are used by a process dynamics (a sub-class of :class:`Dynamics`)
-    to actually run a simulation of the process.
+class Process():
+    """A process that runs over a network. This is the abstract base class
+    for all network processes.  It provides the essential routines to
+    build, set-up, run, and extract results from a
+    process. Sub-classes provide the actual behaviour by defining
+    simulation events and attaching them to the network in different
+    ways.
+
+    A process definition is largely declarative, in the sense that it
+    sets up event handlers and their probabilities. These are used by
+    a process dynamics (a sub-class of :class:`Dynamics`) to actually
+    run a simulation of the process.
+
+    The process class includes an interface for interacting with the
+    working network. The basic interface is extended and overridden in
+    different sub-classes that interact with the network in different
+    ways.
 
-    The process class includes an interface for interacting with the working network. The basic
-    interface is extended and overridden in different sub-classes that interact with the network
-    in different ways.
     """
 
     # defaults
     DEFAULT_MAX_TIME = 20000      #: Default maximum simulation time.
 
     def __init__(self):
-        super(Process, self).__init__()
+        super().__init__()
 
         # set the default maximum time, which persists across runs of the process
         self._maxTime = self.DEFAULT_MAX_TIME
@@ -65,15 +72,19 @@ class Process(object):
     # ---------- Setup and initialisation ----------
 
     def reset(self):
-        """Reset the process ready to be built. This resets all the internal process state
-        variables. The default does nothng."""
+        """Reset the process ready to be built. This resets all the internal
+        process state variables. The default does nothng.
+
+        """
         pass
 
     def build(self, params : Dict[str, Any]):
-        """Build the process model. This should be overridden by sub-classes, and should
-        create the various elements of the model.
+        """Build the process model. This should be overridden by sub-classes,
+        and should create the various elements of the model.
 
-        :param params: the model parameters"""
+        :param params: the model parameters
+
+        """
         pass
 
     def setUp(self, params : Dict[str, Any]):
@@ -81,12 +92,16 @@ class Process(object):
         nothing: sub-classes should override it to initialise node states
         or establish other properties ready for the experiment.
 
-        :param params: the simulation parameters"""
+        :param params: the simulation parameters
+
+        """
         pass
 
     def tearDown(self):
-        """Tear down any structures built for this run of the process. The default does
-        nothing."""
+        """Tear down any structures built for this run of the process. The
+        default does nothing.
+
+        """
         pass
 
 
@@ -95,13 +110,17 @@ class Process(object):
     def setNetwork(self, g : Graph):
         """Set the network the process is running over.
 
-        :param g: the network"""
+        :param g: the network
+
+        """
         self._dynamics.setNetwork(g)
 
     def network(self) -> Graph:
         """Return the network the process is running over.
 
-        :returns: the network"""
+        :returns: the network
+
+        """
         return self._dynamics.network()
 
     def setDynamics(self, d : 'Dynamics'):
@@ -113,19 +132,26 @@ class Process(object):
     def dynamics(self) -> 'Dynamics':
         '''Return the instance of :class:`Dynamics` running this process.
 
-        :returns: the dynamics'''
+        :returns: the dynamics
+
+        '''
         return self._dynamics
 
     def setMaximumTime(self, t : float):
-        """Set the maximum default simulation time. The default is given by :attr:`DEFAULT_MAX_TIME`.
-        This is used by :meth:`atEquilibrium` as the default way to determine equilibrium.
+        """Set the maximum default simulation time. The default is given by
+        :attr:`DEFAULT_MAX_TIME`.  This is used by
+        :meth:`atEquilibrium` as the default way to determine
+        equilibrium.
 
-        The maximum time may be slightly exceeded due to the ways in which events are drawn. 
+        The maximum time may be slightly exceeded due to the ways in
+        which events are drawn.
 
-        Setting the maximum time persists across runs of the process, and isn't reset to its
-        default by calling :meth:`reset`.
+        Setting the maximum time persists across runs of the process,
+        and isn't reset to its default by calling :meth:`reset`.
 
-        :param t: the maximum simulation time"""
+        :param t: the maximum simulation time
+
+        """
         self._maxTime = t
 
     def maximumTime(self) -> float:
@@ -135,25 +161,31 @@ class Process(object):
         return self._maxTime
 
 
-   # ---------- Termination and results ----------
+    # ---------- Termination and results ----------
 
     def atEquilibrium(self, t : float) -> bool:
-        """Test whether the process is an equilibrium. The default simply checks whether
-        the simulation time exceeds the maximum given by :meth:`setMaximumTime`, which
-        defaults to an arbitrary value given by :attr:`DEFAULT_MAX_TIME`.
-        Sub-classes may override this to provide a more sensible definition: in many
-        cases it makes sense to keep calling this method as part of any more advanced
-        test, so as to cut-off runaway processes.
+        """Test whether the process is an equilibrium. The default simply
+        checks whether the simulation time exceeds the maximum given
+        by :meth:`setMaximumTime`, which defaults to an arbitrary
+        value given by :attr:`DEFAULT_MAX_TIME`.  Sub-classes may
+        override this to provide a more sensible definition: in many
+        cases it makes sense to keep calling this method as part of
+        any more advanced test, so as to cut-off runaway processes.
 
         :param t: the current simulation time
-        :returns: True if the proceess is now at equilibrium"""
+        :returns: True if the proceess is now at equilibrium
+
+        """
         return (t >= self.maximumTime())
 
     def results(self) -> Dict[str, Any]:
-       """Create and return an empty dict to be filled with experimental results.
-       Sub-classes should extend this method to add results to the dict.
+       """Create and return an empty dict to be filled with experimental
+       results.  Sub-classes should extend this method to add results
+       to the dict.
 
-       :returns: an empty dict for experimental results"""
+       :returns: an empty dict for experimental results
+
+       """
        return dict()
 
 
@@ -167,12 +199,14 @@ class Process(object):
         self.network().add_node(n, **kwds)
 
     def addNodesFrom(self, ns : Iterable[Node], **kwds):
-        """Add all the nodes in the given iterable to the working network. Any keyword arguments are
-        added as node attributes. This works by calling :meth:`addNode` for each
-        element of the iterable.
+        """Add all the nodes in the given iterable to the working network. Any
+        keyword arguments are added as node attributes. This works by
+        calling :meth:`addNode` for each element of the iterable.
 
         :param ns: an iterable collection of nodes
-        :param kwds: (optional) node attributes"""
+        :param kwds: (optional) node attributes
+
+        """
         for n in ns:
             self.addNode(n, **kwds)
 
@@ -183,20 +217,26 @@ class Process(object):
         self.network().remove_node(n)
 
     def removeNodesFrom(self, ns : Iterable[Node]):
-        """Remove all the nodes in the given iterable froim the working network. This works by
-        iteratively calling :meth:`removeNode` for each element of the iterable collection.
+        """Remove all the nodes in the given iterable from the working
+        network. This works by iteratively calling :meth:`removeNode`
+        for each element of the iterable collection.
 
-        :param ns: an iterable collection of nodes"""
+        :param ns: an iterable collection of nodes
+
+        """
         for n in ns:
             self.removeNode(n)
 
     def addEdge(self, n : Node, m : Node, **kwds):
-        """Add an edge between nodes. Any keyword arguments are added as edge attributes.
-        If the endpoint nodes do not already exist then an exception is raised.
+        """Add an edge between nodes. Any keyword arguments are added as edge
+        attributes.  If the endpoint nodes do not already exist then
+        an exception is raised.
 
         :param n: the start node
         :param m: the end node
-        :param kwds: (optional) edge attributes"""
+        :param kwds: (optional) edge attributes
+
+        """
         g = self.network()
         if n not in g:
             raise Exception('No node {n} in network'.format(n = n))
@@ -205,12 +245,14 @@ class Process(object):
         g.add_edge(n, m, **kwds)
 
     def addEdgesFrom(self, es : Iterable[Edge], **kwds):
-        """Add all the edges in the given iterable to the working network. Any keyword arguments are
-        added as node attributes. This works by calling :meth:`addEdge` for each
-        element of the iterable.
+        """Add all the edges in the given iterable to the working network. Any
+        keyword arguments are added as node attributes. This works by
+        calling :meth:`addEdge` for each element of the iterable.
 
         :param es: an iterable collection of edges
-        :param kwds: (optional) node attributes"""
+        :param kwds: (optional) node attributes
+
+        """
         for e in es:
             (n, m) = e
             self.addEdge(n, m, **kwds)
@@ -223,10 +265,13 @@ class Process(object):
         self.network().remove_edge(n, m)
 
     def removeEdgesFrom(self, es : Iterable[Edge]):
-        """Remove all the edges in the given iterable collection from the working network. This works
-        by iteratively calling :meth:`removeEdge` for each edge in the iterable.
+        """Remove all the edges in the given iterable collection from the
+        working network. This works by iteratively calling
+        :meth:`removeEdge` for each edge in the iterable.
 
-        :param es: an iterable collection of edges"""
+        :param es: an iterable collection of edges
+
+        """
         for e in es:
             self.removeEdge(*e)
 
@@ -256,33 +301,40 @@ class Process(object):
         return self.loci()[n]
 
     def addEventPerElement(self, l : Union[str, Locus], p : float, ef : EventFunction):
-        """Add a probabilistic event at a locus, occurring with a particular (fixed)
-        probability for each element of the locus, and calling the :term:`event function`
-        when it is selected. The locus may be a :class:`Locus` object or a string, which
-        is taken to be a locus of this process.
+        """Add a probabilistic event at a locus, occurring with a particular
+        (fixed) probability for each element of the locus, and calling
+        the :term:`event function` when it is selected. The locus may
+        be a :class:`Locus` object or a string, which is taken to be a
+        locus of this process.
 
-        Unlike fixed probability events added by :meth:`addFixedRateEvent`, a per-element
-        event happens with the same probability to each element in the locus. This leads to
-        larger loci giving rise to a higher rate of events.
+        Unlike fixed probability events added by
+        :meth:`addFixedRateEvent`, a per-element event happens with
+        the same probability to each element in the locus. This leads
+        to larger loci giving rise to a higher rate of events.
 
         :param l: the locus or locus name
         :param p: the event probability
-        :param ef: the event function"""
+        :param ef: the event function
+
+        """
         self._dynamics.addEventPerElement(self, l, p, ef)
 
     def addFixedRateEvent(self, l : Union[str, Locus], p : float, ef : EventFunction):
-        """Add a probabilistic event at a locus, occurring with a particular (fixed)
-        probability, and calling the :term:`event function`
-        when it is selected. The locus may be a :class:`Locus` object or a string, which
-        is taken to be the name of a locus of this process.
+        """Add a probabilistic event at a locus, occurring with a particular
+        (fixed) probability, and calling the :term:`event function`
+        when it is selected. The locus may be a :class:`Locus` object
+        or a string, which is taken to be the name of a locus of this
+        process.
 
-        Unlike fixed rate events added by :meth:`addEventPerElement`, a fixed probability
-        event happens with the same probability regardless of how many elements are in
-        the locus.
+        Unlike fixed rate events added by :meth:`addEventPerElement`,
+        a fixed probability event happens with the same probability
+        regardless of how many elements are in the locus.
 
         :param l: the locus or locus name
         :param p: the event probability
-        :param ef: the event function"""
+        :param ef: the event function
+
+        """
         self._dynamics.addFixedRateEvent(self, l, p, ef)
 
     def postEvent(self, t : float, e : Any, ef : EventFunction):
@@ -299,11 +351,7 @@ class Process(object):
         :param t: the start time
         :param dt: the interval
         :param e: the element (node or edge) on which the event occurs
-        :param ef: the element function"""
+        :param ef: the element function
+
+        """
         self._dynamics.postRepeatingEvent(t, dt, e, ef)
-
-
-
-
-
-
