@@ -17,11 +17,11 @@
 # You should have received a copy of the GNU General Public License
 # along with epydemic. If not, see <http://www.gnu.org/licenses/gpl.html>.
 
-from epydemic import Dynamics, Process, NetworkGenerator
-from networkx import Graph
-import numpy                     # type: ignore
-from copy import copy
 import sys
+from copy import copy
+import numpy                     # type: ignore
+from networkx import Graph
+from epydemic import Dynamics, Process, NetworkGenerator
 if sys.version_info >= (3, 8):
     from typing import Any, Dict, Union, Final
 else:
@@ -29,6 +29,7 @@ else:
     from typing import Any, Dict, Union
     from typing_extensions import Final
 
+
 class SynchronousDynamics(Dynamics):
     '''A dynamics that runs synchronously in discrete time, applying local
     rules to each node in the network. These are simple to understand and
@@ -39,12 +40,12 @@ class SynchronousDynamics(Dynamics):
     :param g: prototype network to run over (optional)'''
 
     # additional metadata
-    TIMESTEPS_WITH_EVENTS : Final[str] = 'epydemic.dynamics.timesteps_with_events'  #: Metadata element holding the number timesteps that actually had events occur within them
+    TIMESTEPS_WITH_EVENTS: Final[str] = 'epydemic.dynamics.timesteps_with_events'  #: Metadata element holding the number timesteps that actually had events occur within them
 
-    def __init__(self, p : Process, g : Union[Graph, NetworkGenerator] =None):
+    def __init__(self, p: Process, g: Union[Graph, NetworkGenerator] =None):
         super().__init__(p, g)
 
-    def do(self, params : Dict[str, Any]) -> Dict[str, Any]:
+    def do(self, params: Dict[str, Any]) -> Dict[str, Any]:
         '''Execute the process under synchronous dynamics.
 
         :param params: the parameters of the simulation
@@ -53,10 +54,12 @@ class SynchronousDynamics(Dynamics):
         # run the
         rng = numpy.random.default_rng()
         proc = self.process()
-        t = 0
+        t = 1.0
         events = 0
         timestepEvents = 0
         while not proc.atEquilibrium(t):
+            self.setCurrentSimulationTime(t)
+
             # fire any events posted for at or before this time
             nev = self.runPendingEvents(t)
 
@@ -86,10 +89,13 @@ class SynchronousDynamics(Dynamics):
             events = events + nev
             if nev > 0:
                 # we had events happen in this timestep
-                timestepEvents = timestepEvents + 1
+                timestepEvents += 1
 
             # advance to the next timestep
-            t = t + 1
+            t += 1.0
+
+        # add topology marker
+        (self.parameters())[NetworkGenerator.TOPOLOGY] = self.networkGenerator().topology()
 
         # add some more metadata
         (self.metadata())[self.TIME] = t
