@@ -1,7 +1,7 @@
 # Test compartmented model basic functions
 #
 # Copyright (C) 2017--2020 Simon Dobson
-# 
+#
 # This file is part of epydemic, epidemic network simulations in Python.
 #
 # epydemic is free software: you can redistribute it and/or modify
@@ -30,7 +30,7 @@ class CompartmentedModelTest(unittest.TestCase):
         self._params[SIR.P_INFECT] = 0.1
         self._params[SIR.P_INFECTED] = 0.01
         self._params[SIR.P_REMOVE] = 0.05
-       
+
     def testPopulation( self ):
         '''Test populating a model.'''
         e = StochasticDynamics(SIR(), self._er)
@@ -60,8 +60,8 @@ class CompartmentedModelTest(unittest.TestCase):
         m.changeCompartmentInitialOccupancy(SIR.INFECTED, pInfected)
         m.changeCompartmentInitialOccupancy(SIR.SUSCEPTIBLE, 1.0 - pInfected)
         m.setUp(self._params)
-        self.assertAlmostEqual(len(m.compartment(SIR.INFECTED)) / self._er.order(), pInfected, places=2)
-        self.assertAlmostEqual(len(m.compartment(SIR.SUSCEPTIBLE)) / self._er.order(), (1.0 - pInfected), places=2)
+        self.assertAlmostEqual(len(m.compartment(SIR.INFECTED)) / self._er.order(), pInfected, places=1)
+        self.assertAlmostEqual(len(m.compartment(SIR.SUSCEPTIBLE)) / self._er.order(), (1.0 - pInfected), places=1)
 
     def testMissingCompartment(self):
         '''Test we catch missing compartments when changing initial occupancy.'''
@@ -72,7 +72,7 @@ class CompartmentedModelTest(unittest.TestCase):
         m.build(self._params)
         with self.assertRaises(Exception):
             m.changeCompartmentInitialOccupancy('missing', 1.0)
-         
+
     def testSanityCheckDistribution(self):
         '''Check that we detect a bad distribution in initial comaprtment occupancy.'''
         m = SIR()
@@ -126,22 +126,22 @@ class CompartmentedModelTest(unittest.TestCase):
         # keep track of the other compartments as well
         m.trackNodesInCompartment(SIR.SUSCEPTIBLE)
         m.trackNodesInCompartment(SIR.REMOVED)
-        
+
         # all nodes in I
-        self.assertCountEqual(m.loci()[SIR.INFECTED].elements(), [1, 2, 3, 4])
+        self.assertCountEqual(m.loci()[SIR.INFECTED], [1, 2, 3, 4])
 
         # one node from I into S, two edges into SI
         m.changeCompartment(1, SIR.SUSCEPTIBLE)
-        self.assertCountEqual(m.loci()[SIR.INFECTED].elements(), [2, 3, 4])
-        self.assertCountEqual(m.loci()[SIR.SUSCEPTIBLE].elements(), [1])
-        self.assertCountEqual(m.loci()[SIR.SI].elements(), [(1, 2), (1, 4)])
+        self.assertCountEqual(m.loci()[SIR.INFECTED], [2, 3, 4])
+        self.assertCountEqual(m.loci()[SIR.SUSCEPTIBLE], [1])
+        self.assertCountEqual(m.loci()[SIR.SI], [(1, 2), (1, 4)])
 
         # recover the infected node
         m.changeCompartment(1, SIR.REMOVED)
-        self.assertCountEqual(m.loci()[SIR.INFECTED].elements(), [2, 3, 4])
-        self.assertCountEqual(m.loci()[SIR.SUSCEPTIBLE].elements(), [])
-        self.assertCountEqual(m.loci()[SIR.REMOVED].elements(), [1])
-        self.assertCountEqual(m.loci()[SIR.SI].elements(), [])
+        self.assertCountEqual(m.loci()[SIR.INFECTED], [2, 3, 4])
+        self.assertCountEqual(m.loci()[SIR.SUSCEPTIBLE], [])
+        self.assertCountEqual(m.loci()[SIR.REMOVED], [1])
+        self.assertCountEqual(m.loci()[SIR.SI], [])
 
     def testSkeletonise( self ):
         '''Test that a network skeletonises correctly'''
